#ifndef RT_DX11_DEVICE_OBJECTS_HPP
#define RT_DX11_DEVICE_OBJECTS_HPP

// Types containing various api objects
#include <stdint.h>
#include <d3d11.h>
#include <d3d11_1.h>

#include "runtime/runtime.h"

struct rt_render_target {
    // Only one of these should be valid
    ID3D11RenderTargetView *rtv;
    ID3D11DepthStencilView *dsv;

    ID3D11Texture2D *texture;

    bool IsColorRenderTarget() const {
        RT_ASSERT(!(rtv != nullptr && dsv != nullptr),
                  "A render target should not contain a render target and a depth stencil view");
        return rtv != nullptr;
    }

    bool IsDepthStencilTarget() const {
        RT_ASSERT(!(rtv != nullptr && dsv != nullptr),
                  "A render target should not contain a render target and a depth stencil view");
        return dsv != nullptr;
    }
    
    uint32_t version;
    rt_render_target *next_free;
};

struct rt_command_buffer {
    // Only created once and then re-used.
    ID3D11DeviceContext1 *context;

    uint32_t version;
    rt_command_buffer *next_free;
};

struct rt_buffer {
    ID3D11Buffer *buffer;
    rt_buffer_type type;
    rt_buffer_usage usage;

    uint32_t version;
    rt_buffer *next_free;
};

rt_render_target *rtGetRenderTarget(rt_render_target_handle handle);
rt_command_buffer *rtGetCommandBuffer(rt_command_buffer_handle handle);
rt_buffer *rtGetBuffer(rt_buffer_handle handle);

#endif